﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LogJoint.RegularExpressions;

namespace LogJoint.StreamParsingStrategies
{
	public abstract class SingleThreadedStrategy : BaseStrategy
	{
		public SingleThreadedStrategy(ILogMedia media, Encoding encoding, IRegex headerRe, MessagesSplitterFlags splitterFlags, TextStreamPositioningParams textStreamPositioningParams)
			: base(media, encoding, headerRe, textStreamPositioningParams)
		{
			this.textSplitter = new ReadMessageFromTheMiddleProblem(new MessagesSplitter(new StreamTextAccess(media.DataStream, encoding, textStreamPositioningParams), headerRe, splitterFlags));
		}

		public override void ParserCreated(CreateParserParams p)
		{
			postprocessor = p.Postprocessor;
			textSplitter.BeginSplittingSession(p.Range.Value, p.StartPosition, p.Direction);

			// todo
			//if (textSplitter.CurrentMessageIsEmpty)
			//{
			//    if (direction == MessagesParserDirection.Forward)
			//    {
			//        if ((initParams.startPosition == reader.BeginPosition)
			//         || ((reader.EndPosition - initParams.startPosition) >= StreamTextAccess.MaxTextBufferSize))
			//        {
			//            throw new InvalidFormatException();
			//        }
			//    }
			//    else
			//    {
			//        // todo
			//    }
			//}
		}

		public override void ParserDestroyed()
		{
			textSplitter.EndSplittingSession();
		}

		public override MessageBase ReadNext()
		{
			if (!textSplitter.GetCurrentMessageAndMoveToNextOne(capture))
				return null;
			return MakeMessage(capture);
		}

		public override PostprocessedMessage ReadNextAndPostprocess() 
		{
			var msg = ReadNext();
			return new PostprocessedMessage(msg, (postprocessor != null && msg != null) ? postprocessor(msg) : null);
		}

		protected abstract MessageBase MakeMessage(TextMessageCapture capture);

		IMessagesSplitter textSplitter;
		TextMessageCapture capture = new TextMessageCapture();
		Func<MessageBase, object> postprocessor;
	}
}
