﻿// The following code was generated by Microsoft Visual Studio 2005.
// The test owner should check each test for validity.
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Text;
using System.Collections.Generic;
using System.IO;
using LogJoint;

namespace LogJointTests
{
	[TestClass()]
	public class ConcatReadingStreamTest
	{
		MemoryStream CreateCharStm(char start, char end)
		{
			StringBuilder buf = new StringBuilder();
			for (; start != end; ++start)
				buf.Append(start);
			MemoryStream ms = new MemoryStream(Encoding.ASCII.GetBytes(buf.ToString()));
			ms.Position = 0;
			return ms;
		}

		ConcatReadingStream Create(params MemoryStream[] streams)
		{
			ConcatReadingStream ret = new ConcatReadingStream();
			ret.Update(streams);
			return ret;
		}

		void CheckRead(ConcatReadingStream stm, string expectation)
		{
			CheckRead(stm, expectation.Length, expectation);
		}

		void CheckRead(ConcatReadingStream stm, int bytesToRead, string expectation)
		{
			byte[] buf = new byte[bytesToRead];
			long savePos = stm.Position;
			int read = stm.Read(buf, 0, bytesToRead);
			Assert.AreEqual(expectation.Length, read);
			Assert.AreEqual(expectation, Encoding.ASCII.GetString(buf, 0, read));
			Assert.AreEqual(savePos + read, stm.Position);
		}

		[TestMethod()]
		[ExpectedException(typeof(ArgumentOutOfRangeException))]
		public void Position_NegativeIsNotAllowed()
		{
			Create(CreateCharStm('a', 'b')).Position = -1;
		}

		[TestMethod()]
		public void Read_SingleStream_ExactlyWholeStream()
		{
			CheckRead(Create(CreateCharStm('a', 'g')), "abcdef");
		}

		[TestMethod()]
		public void Read_SingleStream_ReadMoreBytesThanExists()
		{
			CheckRead(Create(CreateCharStm('a', 'g')), 100, "abcdef");
		}

		[TestMethod()]
		public void Read_SingleStream_ReadFromMiddleOfTheStream()
		{
			ConcatReadingStream s = Create(CreateCharStm('a', 'g'));
			s.Position = 3;
			CheckRead(s, "def");
		}

		[TestMethod()]
		public void Read_SingleStream_ReadFromMiddleOfTheStream_ReadMoreThanExists()
		{
			ConcatReadingStream s = Create(CreateCharStm('a', 'g'));
			s.Position = 4;
			CheckRead(s, 100, "ef");
		}

		[TestMethod()]
		public void Read_SingleStream_ReadExactlyFromTheEndOfTheStream()
		{
			ConcatReadingStream s = Create(CreateCharStm('a', 'g'));
			s.Position = 6;
			CheckRead(s, 100, "");
		}

		[TestMethod()]
		public void Read_SingleStream_ReadFromThePositionAfterTheEnd()
		{
			ConcatReadingStream s = Create(CreateCharStm('a', 'g'));
			s.Position = 10;
			CheckRead(s, 100, "");
		}

		[TestMethod()]
		public void Read_TwoStreams_ReadExactlyWholeStream()
		{
			ConcatReadingStream s = Create(
				CreateCharStm('a', 'g'),
				CreateCharStm('o', 'u')
			);
			CheckRead(s, 100, "abcdefopqrst");
		}

		[TestMethod()]
		public void Read_TwoStreams_ReadAllFromTheMiddleOfTheFirstStream()
		{
			ConcatReadingStream s = Create(
				CreateCharStm('0', '7'),
				CreateCharStm('o', 'u')
			);
			s.Position = 4;
			CheckRead(s, 100, "456opqrst");
		}

		[TestMethod()]
		public void Read_TwoStreams_ReadFromTheMiddleOfTheFirstStreamToTheMiddleOfTheSecondStream()
		{
			ConcatReadingStream s = Create(
				CreateCharStm('0', '7'),
				CreateCharStm('o', 'u')
			);
			s.Position = 4;
			CheckRead(s, 7, "456opqr");
		}

		[TestMethod()]
		public void Read_TwoStreams_ReadFromTheMiddleOfTheSecondStream()
		{
			ConcatReadingStream s = Create(
				CreateCharStm('0', '7'),
				CreateCharStm('o', 'u')
			);
			s.Position = 8;
			CheckRead(s, 2, "pq");
		}

		[TestMethod()]
		public void Read_TwoStreams_ReadNothing()
		{
			ConcatReadingStream s = Create(
				CreateCharStm('0', '7'),
				CreateCharStm('o', 'u')
			);
			s.Position = 8;
			CheckRead(s, 0, "");
			Assert.AreEqual(s.Position, (long)8);
		}

		[TestMethod()]
		public void Read_ThreeStreams_ReadOverTheSteamInTheMiddle()
		{
			ConcatReadingStream s = Create(
				CreateCharStm('0', '7'),
				CreateCharStm('a', 'f'),
				CreateCharStm('o', 'u')
			);
			s.Position = 5;
			CheckRead(s, 10, "56abcdeopq");
		}

	}


}
